#!/bin/bash

source `which setPortNumbers.sh`

PROLOG="BEGIN_PROLOG \
def_offset: 0x2700 \
def_channels: [ { channel: 0 enable: true dc_offset: @local::def_offset }, \
		{ channel: 1 enable: true dc_offset: @local::def_offset }, \
		{ channel: 2 enable: true dc_offset: @local::def_offset }, \
		{ channel: 3 enable: true dc_offset: @local::def_offset }, \
		{ channel: 4 enable: true dc_offset: @local::def_offset }, \
		{ channel: 5 enable: true dc_offset: @local::def_offset }, \
		{ channel: 6 enable: true dc_offset: @local::def_offset }, \
		{ channel: 7 enable: true dc_offset: @local::def_offset } ] \
END_PROLOG"

GENA="generator: V172xDriver \
  generator_ds50: {fragment_id: 0} \
  link_type: PCIE \
  link_number: 4 \
  io_level: TTL \
  acquisition_gate_us: 400 \
  pre_trigger_us: 40 \
  acquisition_mode: HW \
  multiboard_sincronicazion: true \
  channels: @local::def_channels"

GENB="generator: V172xDriver \
  generator_ds50: {fragment_id: 1} \
  link_type: PCIE \
  link_number: 5 \
  io_level: TTL \
  acquisition_gate_us: 400 \
  pre_trigger_us: 40 \
  acquisition_mode: HW \
  multiboard_sincronicazion: true \
  channels: @local::def_channels"

GENC="generator: V172xDriver \
  generator_ds50: {fragment_id: 2} \
  link_type: PCIE \
  link_number: 6 \
  io_level: TTL \
  acquisition_gate_us: 400 \
  pre_trigger_us: 40 \
  acquisition_mode: HW \
  multiboard_sincronicazion: true \
  channels: @local::def_channels"

GEND="generator: V172xDriver \
  generator_ds50: {fragment_id: 3} \
  link_type: PCIE \
  link_number: 7 \
  io_level: TTL \
  acquisition_gate_us: 400 \
  pre_trigger_us: 40 \
  acquisition_mode: HW \
  multiboard_sincronicazion: true \
  channels: @local::def_channels"

GEN1495="generator: V1495Driver \
  generator_ds50: {fragment_id: 4} \
  link_type: PCIE \
  link_number: 0 \
  vme_base: 0x01000000 \
  pulser_frequency: 32768 \
  acquisition_gate_us: 410 \
  memory_full_extension_us: 50"

TIMESTAMP=`date '+%Y%m%d%H%M%S'`
ART_DISK_CFG="physics: { \
  my_output_modules: [ normalOutput ] \
  end_paths: [ my_output_modules ] \
} \
outputs: { \
  normalOutput: { \
    module_type: RootOutput \
    fileName: \"OUTDIR/ds50daq_run000501_ebXX_${TIMESTAMP}.root\" \
    compressionLevel: 0 \
  } \
} \
source: \
{ \
  module_type: RawInput \
  waiting_time: 20 \
  resume_after_timeout: true \
  fragment_type_map: [[1, \"missed\"], [2, \"V1495\"], [3, \"V1720\"], [4, \"V1724\"]] \
} \
process_name: ONLINEDEMO"


xmlrpc http://$readerHost:$frPort0/RPC2 daq.init "$PROLOG daq: {max_fragment_size_words: 524288 fragment_receiver: {mpi_buffer_count: 10 first_event_builder_rank: 5 event_builder_count: 1 $GENA}}" &

xmlrpc http://$readerHost:$frPort1/RPC2 daq.init "$PROLOG daq: {max_fragment_size_words: 524288 fragment_receiver: {mpi_buffer_count: 10 first_event_builder_rank: 5 event_builder_count: 1 $GENB}}" &

xmlrpc http://$readerHost:$frPort2/RPC2 daq.init "$PROLOG daq: {max_fragment_size_words: 524288 fragment_receiver: {mpi_buffer_count: 10 first_event_builder_rank: 5 event_builder_count: 1 $GENC}}" &

xmlrpc http://$readerHost:$frPort3/RPC2 daq.init "$PROLOG daq: {max_fragment_size_words: 524288 fragment_receiver: {mpi_buffer_count: 10 first_event_builder_rank: 5 event_builder_count: 1 $GEND}}" &

xmlrpc http://$readerHost:$frPort4/RPC2 daq.init "$PROLOG daq: {max_fragment_size_words: 524288 fragment_receiver: {mpi_buffer_count: 10 first_event_builder_rank: 5 event_builder_count: 1 $GEN1495}}" &


#artConfig=`echo $ART_CANVAS_CFG | sed 's/YY/0/g'`
artConfig=`echo $ART_DISK_CFG | sed 's/XX/00/g'`
useArt=true
if [[ `echo $builderHost | grep 'fr'` != "" ]]; then
    if [[ `echo $builderHost | grep 'dsfr5'` != "" ]]; then
        artConfig=`echo $artConfig | sed 's,OUTDIR,/scratch/biery,g'`
    else
        artConfig=`echo $artConfig | sed 's,OUTDIR,/data/wh14,g'`
    fi
else
    if [[ `echo $builderHost | grep 'dseb7'` != "" ]]; then
        artConfig=`echo $artConfig | sed 's,OUTDIR,/data1/ds50tb,g'`
    else
        artConfig=`echo $artConfig | sed 's,OUTDIR,/data1/wh14,g'`
    fi
fi
xmlrpc http://$builderHost:$ebPort0/RPC2 daq.init "daq: {max_fragment_size_words: 524288 event_builder: {mpi_buffer_count: 50 first_fragment_receiver_rank: 0 fragment_receiver_count: 5 expected_fragments_per_event: 5 use_art: ${useArt} print_event_store_stats: true}} ${artConfig}" &

wait
